/* <APPLET CODE="Applet3.class" WIDTH=700 HEIGHT=550></APPLET> */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.applet.*;

public class Applet3 extends Applet {
	static ImageViewer clusterViewer;
	static Histogram histogram;
	static List propList;
	static GraphCanvas lightGraph;
	static CalculationPanel calcPanel;

	Color bgColor = Color.darkGray;
	Color activeColor = Color.gray;
	Color inactiveColor = Color.darkGray;

	static Cluster currentCluster;
	static Galaxy currentGalaxy;

	static Button selectButton;
	static Button dataButton;

	Label dataCreditHeader = new Label("Data source:");
	Label imageCreditHeader = new Label("Image source");
	Label dataCreditLabel = new Label("");
	Label imageCreditLabel = new Label("");

	static String codebase;

	CardLayout cardLayout;

	static Choice clusterChoice;

	Panel northPanel;
	Panel centerPanel;
	Panel selectPanel;
	Panel dataPanel;

	int WIDTH = getSize().width;
	int HEIGHT = getSize().height;

	//*****************************************//

	public void start() {
		try {
			bgColor = new Color(Integer.parseInt(this.getParameter("bgColor"), 16));
		} catch (Exception e) {}
		try {
			activeColor = new Color(Integer.parseInt(this.getParameter("activeColor"), 16));
		} catch (Exception e) {}
		try {
			inactiveColor = new Color(Integer.parseInt(this.getParameter("inactiveColor"), 16));
		} catch (Exception e) {}

		imageCreditLabel.setBackground(bgColor);
		dataCreditLabel.setBackground(bgColor);


		setBackground(bgColor);
		cardLayout = new CardLayout();
		clusterViewer = new ImageViewer();
		histogram = new Histogram(2000,12000,50);
		calcPanel = new CalculationPanel();
		propList = new List(6);
		lightGraph = new GraphCanvas();
		northPanel = new Panel();
		centerPanel = new Panel(cardLayout);
		dataPanel= new Panel();
		dataPanel.setBackground(bgColor);
		selectPanel = new Panel();
		selectPanel.setBackground(bgColor);
		selectButton = new Button("Select");
		selectButton.setBackground(activeColor);
		selectButton.setForeground(Color.red);
		dataButton = new Button("Analyze");
		dataButton.setBackground(inactiveColor);

		codebase = getCodeBase().toString();

		makeLayout();

		selectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cardLayout.show(centerPanel,"select");
				selectButton.setBackground(activeColor);
				selectButton.setForeground(Color.red);
				dataButton.setBackground(inactiveColor);
				dataButton.setForeground(Color.black);
			}
		});

		dataButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cardLayout.show(centerPanel,"data");

				dataButton.setBackground(activeColor);
				dataButton.setForeground(Color.red);
				selectButton.setBackground(inactiveColor);
				selectButton.setForeground(Color.black);

				histogram.reset();

				if(currentCluster == null) {
					lightGraph.reset();
					calcPanel.calcButton.setEnabled(false);
				} else {
					for(int i=0; i< currentCluster.galList.size(); i++) {
						if(((Galaxy)(currentCluster.galList.elementAt(i))).isChecked()) {
							histogram.addData(((Galaxy)(currentCluster.galList.elementAt(i))).vel);
						}
					}
					histogram.calculate();

					Galaxy[] tempList = new Galaxy[currentCluster.galList.size()];
					for(int i=0; i< tempList.length; i++) {
						tempList[i] = (Galaxy)(currentCluster.galList.elementAt(i));
					}
					lightGraph.setDataArray(tempList);
					calcPanel.calcButton.setEnabled(true);
				}
			}
		});

		clusterChoice.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				dataCreditHeader.setVisible(false);
				imageCreditHeader.setVisible(false);
				if(propList.getItemCount() > 0) propList.removeAll();
				if(clusterChoice.getSelectedIndex() <=0) {
					clusterViewer.setCluster(null);
					currentCluster = null;
					dataCreditLabel.setText("");
					imageCreditLabel.setText("");
					return;
				}
				loadCluster(clusterChoice.getSelectedItem());
			}
		});
	}


	public void ListClusters() {
		try {
			clusterChoice.addItem("Select a cluster");
			String lineOfText;
			URL url = new URL(codebase + "/data/clusters.txt");
			URLConnection urlConn;
			DataInputStream dis;
			urlConn = url.openConnection();
			urlConn.setDoInput(true);
			urlConn.setUseCaches(false);

			dis = new DataInputStream(urlConn.getInputStream());


			BufferedReader reader = new BufferedReader(new InputStreamReader(dis));
			while( (lineOfText = reader.readLine()) != null ) {
				clusterChoice.addItem(lineOfText);
			}
			dis.close();
		} catch (Exception e) {
			System.out.println("Unable to open clusters.txt");
			System.out.println(e);
		}

	}

	public void loadCluster(String name) {
		currentCluster = new Cluster(name);
		clusterViewer.setCluster(currentCluster);
		try {
			dataCreditLabel.setText(currentCluster.dataCredit);
			imageCreditLabel.setText(currentCluster.imageCredit);
			imageCreditHeader.setVisible(true);
			dataCreditHeader.setVisible(true);
		} catch (NullPointerException e) {
			dataCreditLabel.setText("");
			imageCreditLabel.setText("");
			imageCreditHeader.setVisible(false);
			dataCreditHeader.setVisible(false);
		}
	}

	public static void setCurrentGalaxy(Galaxy gal) {
		if(propList.getItemCount() > 0) {
			propList.removeAll();
		} if(gal != null) {
			currentGalaxy = gal;
			propList.add("Number: " + currentGalaxy.number);
			propList.add("Name: " + currentGalaxy.name);
			propList.add("Mag: " + currentGalaxy.mag);
			propList.add("Vel: " + currentGalaxy.vel + " km/sec");
			propList.add("Rad: " + FMath.roundOff(currentGalaxy.rad,3) + " arcsec");

			if(currentGalaxy.isChecked()) propList.add("Is in the data set");
		}
	}

	public void makeLayout() {
		removeAll();
		setLayout(null);

		add(centerPanel);
		centerPanel.setBounds(0,35,700,565);

		add(selectButton);
		selectButton.setBounds(0,5,120,30);

		add(dataButton);
		dataButton.setBounds(130,5,120,30);

		selectPanel.setLayout(null);
		selectPanel.add(clusterViewer);
		clusterViewer.setBounds(5,5,500,500);

		clusterChoice = new Choice();
		ListClusters();
		selectPanel.add(clusterChoice);
		clusterChoice.setBounds(510,5,150,35);

		selectPanel.add(propList);
		propList.setBounds(510,45,150,120);
		propList.setBackground(Color.lightGray);

		selectPanel.add(dataCreditLabel);
		selectPanel.add(imageCreditLabel);
		selectPanel.add(dataCreditHeader);
		selectPanel.add(imageCreditHeader);

		imageCreditHeader.setVisible(false);
		dataCreditHeader.setVisible(false);

		dataCreditHeader.setBounds(510,180,150,30);
		dataCreditLabel.setBounds(510,210,180,30);

		dataCreditHeader.setFont(new Font("Times",Font.PLAIN,11));
		dataCreditLabel.setFont(new Font("Times",Font.PLAIN,11));
		imageCreditHeader.setFont(new Font("Times",Font.PLAIN,11));
		imageCreditLabel.setFont(new Font("Times",Font.PLAIN,11));

		imageCreditHeader.setBounds(510,245,150,30);
		imageCreditLabel.setBounds(510,275,180,30);

		dataPanel.setLayout(null);

		dataPanel.add(histogram);
		histogram.setBounds(5,5,340,340);

		dataPanel.add(lightGraph);
		lightGraph.setBounds(355,5,340,340);

		dataPanel.add(calcPanel);
		calcPanel.setBounds(5,350,690,140);

		centerPanel.add(selectPanel,"select");
		centerPanel.add(dataPanel,"data");
	}



}
