import java.awt.*;
import java.awt.event.*;

public class CalculationPanel extends Panel {

	Button calcButton;
	public static TextField h0InputField, calcMinInputField, calcMaxInputField;


	double G = 4.43e-3;
	double solarLuminosity = 4.74;

	Label distanceLabel;
	Label luminosityLabel;
	Label hlradLabel;
	Label massLabel;
	Label mlLabel;

	Label invalidH0Label= new Label();


	public CalculationPanel() {

		setBackground(Color.black);

		calcButton = new Button("Calculate");
		calcButton.setForeground(Color.white);
		calcButton.setBackground(Color.darkGray);

		h0InputField = new TextField();
		calcMinInputField = new TextField();
		calcMaxInputField = new TextField();

		h0InputField.setBackground(Color.black);
		calcMinInputField.setBackground(Color.black);
		calcMaxInputField.setBackground(Color.black);

		h0InputField.setForeground(Color.white);
		calcMinInputField.setForeground(Color.white);
		calcMaxInputField.setForeground(Color.white);

		calcButton.setBackground(Color.darkGray);

		distanceLabel = new Label();
		distanceLabel.setForeground(Color.white);
		luminosityLabel = new Label();
		luminosityLabel.setForeground(Color.white);
		hlradLabel = new Label();
		hlradLabel.setForeground(Color.white);
		massLabel = new Label();
		massLabel.setForeground(Color.white);
		mlLabel = new Label();
		mlLabel.setForeground(Color.white);

		makeLayout();

		calcButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					if((Double.valueOf(calcMinInputField.getText()).doubleValue() == 0)||(Double.valueOf(calcMinInputField.getText()).doubleValue() == 0)) throw new NumberFormatException();
					Applet3.histogram.setCalcBounds(Double.valueOf(calcMinInputField.getText()).doubleValue(),Double.valueOf(calcMaxInputField.getText()).doubleValue());
				} catch (NumberFormatException nfe) {
					calcMinInputField.setText(Applet3.histogram.calcMin+"");
					calcMaxInputField.setText(Applet3.histogram.calcMax+"");
				}

				try {
					if((Double.valueOf(h0InputField.getText().trim())).doubleValue() <= 0) throw new NumberFormatException();
					invalidH0Label.setText("");
					Applet3.histogram.calculate();
					calculate();
				} catch (NumberFormatException nfe2) {
					invalidH0Label.setText("Enter a Hubble Constant");
				}
			}
		});

	}

	public void reset() {
		h0InputField = new TextField();
		calcMinInputField = new TextField();
		calcMaxInputField = new TextField();

		distanceLabel = new Label();
		luminosityLabel = new Label();
		hlradLabel = new Label();
		massLabel = new Label();
		mlLabel = new Label();
	}


	public void calculate() {
		double h0 = (Double.valueOf(h0InputField.getText())).doubleValue();

		//System.out.println(h0);

		// calculate cluster distance
		double clusterDistance = Applet3.histogram.getMean()/h0;
		//System.out.println(clusterDistance);
		distanceLabel.setText("Cluster Distance: " + FMath.sigDig(clusterDistance,3) + " Mpc");

		// calculate half-light radius
		double hlrad = Applet3.lightGraph.getHlrad();
		hlrad = hlrad * clusterDistance/206265;
		//System.out.println("Half-Light Radius: " + hlrad + " Mpc");
		hlradLabel.setText("Half-Light Radius: " + FMath.sigDig(hlrad,3) + " Mpc");

		// calculate mass of cluster
		double mass = ( 10.0e6 * hlrad * Math.pow(Applet3.histogram.getSigma(),2) ) / G;
		//System.out.println("Mass: " + mass + " solar masses");
		massLabel.setText("Mass: " + FMath.sigDig(mass,3) + " solar masses");

		//System.out.println(G);

		// calculate the luminosity in solar luminosities
		double luminosity = ( ((Math.pow(clusterDistance*1000000,2))/100.0) * Math.pow(10.0,0.4*solarLuminosity))*Applet3.lightGraph.getTotalLuminosity();
		//System.out.println("Luminosity: " + luminosity + " solar luminosities");
		luminosityLabel.setText("Luminosity: " + FMath.sigDig(luminosity,3) + " solar luminosities");

		// calculate the m/l ratio
		mlLabel.setText("M/L ratio: " + FMath.sigDig( (mass/luminosity) , 3));

	}

	public void makeLayout() {
		setLayout(null);

		Label h0Label = new Label("Hubble Constant:");
		h0Label.setForeground(Color.white);

		Label calcMinLabel = new Label("Lower bound of dispersion calculation:");
		Label calcMaxLabel = new Label("Upper bound of dispersion calculation:");
		calcMinLabel.setForeground(Color.white);
		calcMaxLabel.setForeground(Color.white);

		add(calcMinLabel);
		calcMinLabel.setBounds(0,5,240,25);

		add(calcMinInputField);
		calcMinInputField.setBounds(250,3,90,30);

		add(calcMaxLabel);
		calcMaxLabel.setBounds(0,35,240,25);

		add(calcMaxInputField);
		calcMaxInputField.setBounds(250,33,90,30);


		add(h0Label);
		h0Label.setBounds(0,60,110,25);

		add(h0InputField);
		h0InputField.setBounds(110,60,40,30);

		add(invalidH0Label);
		invalidH0Label.setBounds(160,60,180,30);
		invalidH0Label.setForeground(Color.red);

		add(calcButton);
		calcButton.setBounds(0,90,150,25);

		add(distanceLabel);
		distanceLabel.setBounds(350,0,350,25);

		add(luminosityLabel);
		luminosityLabel.setBounds(350,30,350,25);

		add(hlradLabel);
		hlradLabel.setBounds(350,60,350,25);

		add(massLabel);
		massLabel.setBounds(350,90,350,25);

		add(mlLabel);
		mlLabel.setBounds(350,120,350,25);



	}




}
