import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.ImageFilter.*;
import java.util.*;

/**
*	A JDK1.1-compatible plotting frame
*/

public class GraphCanvas extends Canvas {
	/** The maximum value on the horizontal axis in the visible graph **/
	public double xMax=0;
	/** The minimum value on the horizontal axis in the visible graph **/
	public double xMin=0;
	/** The maximum value on the vertical axis in the visible graph **/
	public double yMax=0;
	/** The minimum value on the vertical axis in the visible graph **/
	public double yMin=0;

	int xEdge=15, yEdge=15;



	/**	The text label for the horizontal axis **/
	public String xLabel = new String();
	/** The text label for the vertical axis **/
	public String yLabel = new String();

	Image labelImage,transformedImage;


	double totalLuminosity=0;

	/** An array to hold data points in rad,lum format **/
	double[][] dataArray= new double[0][2];

	double hlrad;

	/**
	*	The width of the graph in pixels
	*	currently hard-coded due to incompatibility issues
	**/
	int WIDTH;
	/**
	*	The height of the graph in pixels
	*	currently hard-coded due to incompatibility issues
	**/
	int HEIGHT;


	/** Initializes a new GraphPane **/
	public GraphCanvas() {
		this.setBackground(Color.black);
		repaint();
	}


	public void rotateLabels() {
		if(labelImage != null) {
			Graphics h = labelImage.getGraphics();
			ImageFilter rotateFilter = new RotateFilter(Math.PI/2);
			h.drawString("Fractional Luminosity",5,20);
			ImageProducer p = new FilteredImageSource(labelImage.getSource(),rotateFilter);
			transformedImage = createImage(p);
		}
	}

	public double getHlrad() {
		 return(hlrad);
	}


	/** Paints the GraphCanvas **/
	public void paint(Graphics g) {
		
		g.setFont(new Font("Courier",Font.PLAIN,10));

		g.setColor(Color.gray);
		int screenX, screenY;
		for(double i=0; i<xMax; i+=200.0) {
			g.setColor(Color.gray);
			screenX = (int)(xEdge+((getSize().width-xEdge)/(xMax-xMin))*(i - xMin));
			g.drawLine(screenX,0,screenX,getSize().height-yEdge);
			g.setColor(Color.white);
			g.drawString(Math.round(i)+"",screenX-9,getSize().height-yEdge-2);
		}
		g.drawString("Radius (arcsec)",getSize().width/2-30,getSize().height-1);

		for(double i=0; i<=1; i+=.25) {
			g.setColor(Color.gray);
			screenY = (int)(((getSize().height-yEdge)) - ( ((getSize().height-yEdge)/(yMax-yMin))*(i - yMin)));
			g.drawLine(xEdge,screenY,getSize().width,screenY);
			g.setColor(Color.white);
			if(i>0) g.drawString(i+"",xEdge+2,screenY+10);
		}
		g.drawString("Fractional",5,10);
		g.drawString("Luminosity",5,18);
		
		if(Applet3.currentCluster == null) return;




		g.setColor(Color.yellow);
		Point p1, p2;
		for(int i=1; i< dataArray.length; i++) {
			p1 = dataToScreen(dataArray[i-1][0],dataArray[i-1][1]);
			p2 = dataToScreen(dataArray[i][0],dataArray[i][1]);
			g.drawLine(p1.x,p1.y,p2.x,p2.y);
		}

		g.setColor(Color.white);
		
		if(hlrad > 0) {
			for(double i=(0.5/50); i<=.5; i+=(0.5/25)) {
				p1 = dataToScreen(hlrad,i-(0.5/50));
				p2 = dataToScreen(hlrad,i);
				g.drawLine(p1.x,p1.y,p2.x,p2.y);
			}
			p1 = dataToScreen(hlrad,.5);
			g.drawString(FMath.sigDig(hlrad,3) + " arcsec",p1.x+2,p1.y+10);
		}
	}

	public double getTotalLuminosity() {
		return(totalLuminosity);
	}

	public void reset() {
		dataArray = new double[0][2];
		hlrad = 0;
	}

	public void setDataArray(Galaxy[] galArray) {
		dataArray = new double[galArray.length][2];
		for(int i=0; i< galArray.length; i++) {
			if(galArray[i].isChecked()) {
				dataArray[i][0] = galArray[i].rad;
				dataArray[i][1] = Math.pow(10.0, -0.4*galArray[i].mag);
			}
		}
		// sort according to R

		double[] holder;
		for(int i=0; i< dataArray.length; i++) {
			int mindex=i;
			for(int j=i; j<dataArray.length; j++) {
				if(dataArray[j][0] < dataArray[mindex][0]) {
					mindex = j;
				}
			}

			holder = dataArray[mindex];
			dataArray[mindex] = dataArray[i];
			dataArray[i] = holder;
		}

		for (int i=1; i < dataArray.length; i++) {
			dataArray[i][1] += dataArray[i-1][1];
			if(i == dataArray.length-1) {
				totalLuminosity = dataArray[i][1];
			}
		}
		for (int i=1; i < dataArray.length; i++) {
			dataArray[i][1] /= dataArray[dataArray.length-1][1];

			// Find the half light radius
			if((dataArray[i][1] >= 0.5)&&(dataArray[i-1][1] <= 0.5)) {
				// make a point slope form of the line using the two points
				// then find the intercept of that line with the horizontal line
				// at .5
				double dx = dataArray[i][0]-dataArray[i-1][0];
				double dy = dataArray[i][1]-dataArray[i-1][1];

				hlrad = (dataArray[i-1][0] + (dx/dy) * (0.5 - dataArray[i-1][1]));
			}
		}



		xMax = dataArray[dataArray.length-1][0];
		yMax = 1.1;


	}

	/** Given the x,y coordinates of a data point, converts
	*	them to an x,y point in the graphing pane's coordinate
	*	system.
	**/
	public Point dataToScreen(double x, double y) {
		WIDTH = this.getSize().width;
		HEIGHT = this.getSize().height;
		int screenX = 0,screenY = 0;
		double deltaX = xMax - xMin;
		double deltaY = yMax - 0;
		screenX = (int)(xEdge+((WIDTH-xEdge)/deltaX)*(x - xMin));
		screenY = (int)((HEIGHT-yEdge) - (((HEIGHT-yEdge)/deltaY)*(y - 0)));
		return(new Point(screenX,screenY));
	}

	/**	Given the x,y coorinates of a point on the graphing
	*	pane's coordinate system, converts them to an x,y
	*	point in the data's coordinate system
	**/
	public double[] screenToData(Point inputPoint) {
		double[] output = new double[2];
		double dataX =0, dataY =0;
		// WIDTH = this.getSize().getWidth();
		//  HEIGHT = this.getSize().getHeight();
		double deltaX = xMax - xMin;
		double deltaY = yMax - yMin;
		dataX = inputPoint.x*(deltaX/WIDTH)+xMin;
		dataY = ((-inputPoint.y*deltaY)+(HEIGHT*deltaY)+(HEIGHT*yMin))/HEIGHT;
		output[0] = dataX;
		output[1] = dataY;
		return(output);
	}


	/**	Set the horizontal axis label to s **/
	public void setXLabel(String s) {
		xLabel = s;
	}

	/**	Set the vertical axis label to s **/
	public void setYLabel(String s) {
		yLabel = s;
	}

	/**	Set the minimum value on the horizontal axis **/
	public void setXMin(double xMin) {
		this.xMin = xMin;
	}

	/**	Set the maximum value on the horizontal axis **/
	public void setXMax(double xMax) {
		this.xMax = xMax;
	}

	/**	Set the minimum value on the vertical axis **/
	public void setYMin(double yMin) {
		this.yMin = yMin;
	}

	/**	Set the maximum value on the vertical axis **/
	public void setYMax(double yMax) {
		this.yMax = yMax;
	}

}
