import java.awt.*;
import java.awt.event.*;

/**
*	The histogram should display all the bars within its xmin and xmax.
*	However, standard deviation, mean, and amplitude will be calculated
*	using only those values between calcmin and calcmax.
**/

public class Histogram extends Canvas {

	int WIDTH;
	int HEIGHT;

	int xEdge=15, yEdge=15;

	int counter=0;

	double xMax, xMin, yMax;

	double mean, sigma, amplitude;
	double calcMin=1000, calcMax=13000;

	/** The number of bars that will be displayed in this histogram **/
	int resolution;

	/** Array to keep trace of the number of data elements in each chunk of the histogram **/
	int[] countArray;
	double[] binArray;
	double[] dataArray = new double[0];


	public Histogram(double xMin, double xMax, int res) {
		setBackground(Color.black);
		this.xMax = xMax;
		this.xMin = xMin;
		this.resolution = res;
		countArray = new int[res];
		binArray = new double[res];

		int index = 0;
		for(double i=xMin+(xMax-xMax)/(2*res); i< xMax; i+= (xMax-xMin)/res) {
			binArray[index] = i;
			index++;
		}
		WIDTH = getSize().width;
		HEIGHT = getSize().height;

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				if(e.isShiftDown()) {
					System.out.println(screenToData(e.getX(),1)[0] + "   " + screenToData(1,e.getY())[1]);
				}
			}
		});
	}

	/**
	*	Add a data point to this histogram.
	**/

	public void addData(double value) {
		if((value < xMin)||(value > xMax)) {
			return;
		}
		for(int i=0; i<resolution; i++) {
			if((value > xMin+(double)i/(double)resolution*(xMax-xMin))&&(value < xMin+(double)(i+1)/(double)resolution*(xMax-xMin))) {
				countArray[i] += 1;
				yMax = Math.max(yMax,countArray[i]+1);
				//break;
			}
		}
		counter++;

		double[] tempArray = (double[])dataArray.clone();
		dataArray = new double[dataArray.length+1];
		for(int i=0; i< tempArray.length; i++) {
			dataArray[i] = tempArray[i];
		}
		dataArray[dataArray.length-1] = value;
	}


	public void calculate() {
		// create a temporary array to hold only the values of dataArray between calcmin and calcmax
		int validCount = 0;
		for(int i=0; i< dataArray.length; i++) {
			if((dataArray[i] >= calcMin)&&(dataArray[i] <= calcMax)) {
				validCount++;
			}
		}

		double[] tempArray = new double[validCount];
		int k=0;
		for(int j=0; j< dataArray.length; j++) {
			if((dataArray[j] >= calcMin)&&(dataArray[j] <= calcMax)) {
				tempArray[k] = dataArray[j];
				k++;
			}
		}

		sigma = FMath.standardDeviation(tempArray);
		mean = FMath.mean(tempArray);

		double logAmp=0;
		int validBinCount=0;

		for(int i=0; i< countArray.length; i++) {
			if((binArray[i] >= calcMin)&&(binArray[i] <= calcMax)&&(countArray[i] != 0)) {
				validBinCount++;
				logAmp += Math.log(countArray[i]) + ((Math.pow((binArray[i]-mean),2))/(2*sigma*sigma));
			}
		}
		logAmp /= validBinCount;
		amplitude = Math.exp(logAmp);

		repaint();

	}

	public double getMean() {
		return(mean);
	}

	public double getSigma() {
		return(sigma);
	}

	public void reset() {
		countArray = new int[resolution];
		dataArray = new double[0];
		yMax=0;
		mean = sigma = amplitude = 0;
		repaint();
		int index=0;
		for(double i=xMin+(xMax-xMax)/(2*resolution); i< xMax; i+= (xMax-xMin)/resolution) {
			binArray[index] = i;
			index++;
		}
	}



	public void setResolution(int res) {
		this.resolution = res;
	}

	public void setCalcBounds(double min, double max) {
		this.calcMin = min;
		this.calcMax = max;
	}

	/**
	*
	**/
	public void paint(Graphics g) {
		WIDTH = getSize().width;
		HEIGHT = getSize().height;
		Point p1, p2;
		g.setFont(new Font("Courier",Font.PLAIN,10));
		g.setColor(Color.gray);
		for(int i=0; i<yMax; i+=2) {
			p1= dataToScreen(xMin,i);
			p2 = dataToScreen(xMax,i);
			g.drawLine(p1.x,p1.y,p2.x,p2.y);
		}

		for(double i=xMin; i< xMax; i+=1000) {
			p1= dataToScreen(i,0);
			p2 = dataToScreen(i,yMax);
			g.drawLine(p1.x,p1.y,p2.x,p2.y);
		}

		if((Applet3.currentCluster == null)||(dataArray.length <= 0)) return;

		g.setColor(Color.yellow);
		for(double i=(yMax/20.0) ; i<=yMax; i+=(yMax/10.0)) {
			p1 = dataToScreen(calcMin,i-(yMax/20.0));
			p2 = dataToScreen(calcMin,i);
			g.drawLine(p1.x,p1.y,p2.x,p2.y);
			p1 = dataToScreen(calcMax,i-(yMax/20.0));
			p2 = dataToScreen(calcMax,i);
			g.drawLine(p1.x,p1.y,p2.x,p2.y);
		}


		g.setColor(Color.blue);
		for(double i=0; i< resolution; i++) {
			int y = (int)((countArray[(int)i]/yMax)*(HEIGHT-yEdge));
			g.fillRect(dataToScreen(binArray[(int)i],0).x,(HEIGHT-yEdge)-y,(WIDTH-xEdge)/resolution-1,y);
		}

		g.setColor(Color.red);

		double value;
		for(int i=1; i< binArray.length; i++) {
			value = Math.exp(-(((binArray[i-1])-mean)*((binArray[i-1])-mean))/(2*sigma*sigma));
			p1 = dataToScreen((binArray[i-1]),amplitude*(value));
			value = Math.exp(-(((binArray[i])-mean)*((binArray[i])-mean))/(2*sigma*sigma));
			p2 = dataToScreen((binArray[i]),amplitude*(value));
			g.drawLine(p1.x,p1.y,p2.x,p2.y);
		}

		g.setColor(Color.white);

		for(int i=2; i<yMax; i+=2) {
			p1= dataToScreen(xMin,i);
			g.drawString(i+"",p1.x+3,p1.y+4);
		}

		for(double i=xMin; i< xMax; i+=1000) {
			p1= dataToScreen(i,0);
			g.drawString(Math.round(i)+"",p1.x-10,p1.y-3);
		}


		g.drawString("Galaxy Velocity (km/sec)",dataToScreen((xMax/2),0).x-30,HEIGHT-2);
		g.drawString("Velocity Dispersion = " + (int)sigma + " km/sec",100,15);
		g.drawString("Mean Velocity = " + (int)mean+" km/sec",100,30);
	}

	public Point dataToScreen(double x, double y) {
		int screenX = 0,screenY = 0;
		double deltaX = xMax - xMin;
		double deltaY = yMax - 0;
		screenX = (int)(xEdge+((WIDTH-xEdge)/deltaX)*(x - xMin));
		screenY = (int)((HEIGHT-yEdge) - (((HEIGHT-yEdge)/deltaY)*(y - 0)));
		return(new Point(screenX,screenY));
	}

	public double[] screenToData(int screenX, int screenY) {
		double dataX = 0,dataY = 0;
		double deltaX = xMax - xMin;
		double deltaY = yMax - 0;
		//screenX = (int)((WIDTH/deltaX)*(x - xMin));
		dataX = (screenX)/(WIDTH/deltaX)+xMin;
		//screenY = (int)((HEIGHT) - ((HEIGHT/deltaY)*(y - 0)));
		dataY = -((screenY - HEIGHT) / (HEIGHT/deltaY));

		return(new double[]{dataX,dataY});
	}




}
