import java.awt.*;
import java.awt.event.*;

public class ImageViewer extends ScrollPane {

	int WIDTH = getSize().width;
	int HEIGHT = getSize().height;


	Cluster currentCluster;

	double pparcsecX;
	double pparcsecY;
	int xCenter, yCenter, xScreen, yScreen;

	Image clusterImage;
	ImageHolder imageHolder;
	int mouseX=0, mouseY=0;

	public ImageViewer() {
		super(ScrollPane.SCROLLBARS_ALWAYS);
		imageHolder = new ImageHolder();
		this.add(imageHolder);

		getHAdjustable().addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent e) {
				imageHolder.repaint();
			}
		});
		getVAdjustable().addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent e) {
				imageHolder.repaint();
			}
		});
	}

	public void setCluster(Cluster cluster) {
		currentCluster = cluster;
		if(currentCluster != null) {
			imageHolder.setSize(currentCluster.imageSize);
			clusterImage = currentCluster.getImage();
		}
		else {
			clusterImage = null;

		}
		imageHolder.repaint();

	}

	/** An internal class used as a canvas for painting the image onto **/

	class ImageHolder extends Canvas {

		Galaxy tempGal;
		boolean overOld=false, overNew;

		public void paint(Graphics g) {

			if(currentCluster == null) {
				g.drawString("Credits:",100,230);
				g.drawString("Scientific Development: Chris Mihos (CWRU)",100,245);
				g.drawString("Programming: Rob Falck (CWRU)",100,260);
				return;
			}

			setSize(currentCluster.imageSize);

			g.drawImage(clusterImage,0,0,this);

			g.setColor(Color.green);
			for(int i=0 ; i< currentCluster.galList.size(); i++) {
				if(((Galaxy)currentCluster.galList.elementAt(i)).isChecked()==true) {
					g.drawOval(((Galaxy)currentCluster.galList.elementAt(i)).x-5,((Galaxy)currentCluster.galList.elementAt(i)).y-5,10,10);
				}
			}
		}

		public ImageHolder() {
			addMouseMotionListener(new MouseMotionAdapter() {
				public void mouseMoved(MouseEvent e) {


					mouseX = e.getX();
					mouseY = e.getY();
					//repaint(10+getScrollPosition().x,5+getScrollPosition().y,80,20);
					if(currentCluster == null) return;
					for(int i=0 ; i< currentCluster.galList.size(); i++) {
						tempGal = (Galaxy)currentCluster.galList.elementAt(i);
						if (FMath.rss((double)(tempGal.x-mouseX),(double)(tempGal.y-mouseY)) < 7) {
							//System.out.println("Mouse is over galaxy " + tempGal.number);
							overNew = true;
							break;
						}
						else if(i== currentCluster.galList.size()-1) {
							tempGal = null;
							overNew = false;
						}
					}
					if(overOld != overNew) {
						repaint(5+getScrollPosition().x,5+getScrollPosition().y,80,35);
						Applet3.setCurrentGalaxy(tempGal);
					}
					overOld = overNew;

				}
			});

			addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					//System.out.println(e);
					String mods = KeyEvent.getKeyModifiersText(e.getModifiers()).toLowerCase();
					if((e.getKeyCode() == 67)&&(mods.indexOf("alt") != -1)&&(mods.indexOf("shift") != -1)&&(mods.indexOf("ctrl") != -1)) {
						for(int i=0; i< currentCluster.galList.size(); i++) {
							((Galaxy)(currentCluster.galList.elementAt(i))).checked = true;

						}
						//System.out.println("All galaxies unchecked");
						repaint();
					}

				}
			});

			addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					mouseX = e.getX();
					mouseY = e.getY();
					if(e.isShiftDown() == true) {
						//for(int i=0; i< currentCluster.galList.size(); i++) {
						//	((Galaxy)(currentCluster.galList.elementAt(i))).checked = true;

						//}
						//System.out.println("All galaxies unchecked");
						//repaint();
					}
					else if(e.isControlDown() == true) {
						for(int i=0; i< currentCluster.galList.size(); i++) {
							((Galaxy)(currentCluster.galList.elementAt(i))).checked = false;

						}
						//System.out.println("All galaxies unchecked");
						repaint();
					}
					else if(tempGal != null) {
						if(tempGal.isChecked()) {
							tempGal.checked = false;
						} else {
							tempGal.checked = true;
						}
						Applet3.setCurrentGalaxy(tempGal);
						repaint(tempGal.x-6, tempGal.y-6,12,12);
					}

				}
			});
		}

	}



}
